@@ -47,13 +47,14 @@ class EncoderLayer(nn.Module):
             num_heads: int,
             dropout_rate: float,
             ff_factory: Callable[[], nn.Module],
-            norm_factory: Callable[[], nn.Module]
+            norm_factory: Callable[[], nn.Module],
+            use_flash: bool = False,
     ) -> None:
         super(EncoderLayer, self).__init__()
 
         self.d_model = d_model
         self.num_heads = num_heads
-        self.mha = MultiheadAttention(d_model, num_heads, dropout_rate)
+        self.mha = MultiheadAttention(d_model, num_heads, dropout_rate, use_flash=use_flash)
 
         self.ff_module = ff_factory()
 
@@ -127,21 +128,27 @@ class DecoderLayer(nn.Module):
             num_heads: int,
             dropout_rate: float,
             ff_factory: Callable[[], nn.Module],
-            norm_factory: Callable[[], nn.Module]
+            norm_factory: Callable[[], nn.Module],
+            use_flash: bool = False,
+            use_cross_attn: bool = False,
     ):
         super(DecoderLayer, self).__init__()
 
-        self.self_attention = MultiheadAttention(d_model, num_heads, dropout_rate)
-        self.cross_attention = MultiheadAttention(d_model, num_heads, dropout_rate)
-
+        self.self_attention = MultiheadAttention(d_model, num_heads, dropout_rate, use_flash=use_flash)
         self.ff_module = ff_factory()
 
         self.dropout1 = nn.Dropout(dropout_rate)
-        self.dropout2 = nn.Dropout(dropout_rate)
         self.norm1 = norm_factory()
-        self.norm2 = norm_factory()
         self.norm3 = norm_factory()
 
+        self.use_cross_attn = use_cross_attn
+        if self.use_cross_attn:
+            self.cross_attention = MultiheadAttention(d_model, num_heads, dropout_rate, use_flash=use_flash)
+            self.dropout2 = nn.Dropout(dropout_rate)
+            self.norm2 = norm_factory()
+        else:
+            self.cross_attention = None
+
     def forward(
             self,
             x: torch.Tensor,
@@ -192,7 +199,7 @@ class DecoderLayer(nn.Module):
             mask=tgt_mask, key_padding_mask=tgt_key_padding_mask)
         out1 = x + self.dropout1(self_attention_output)
 
-        if memory is not None:
+        if self.use_cross_attn and memory is not None:
             cross_attention_output, cross_attention_weights = self.cross_attention(
                 self.norm2(out1), memory, memory, mask=memory_mask, key_padding_mask=memory_key_padding_mask)
             out2 = out1 + self.dropout2(cross_attention_output)
@@ -238,7 +245,8 @@ class Encoder(nn.Module):
             dropout_rate: float,
             ff_factory: Callable[[], nn.Module],
             norm_factory: Callable[[], nn.Module],
-            pad_token_id: int
+            pad_token_id: int,
+            use_flash: bool = False
     ) -> None:
         """
         Initializes the Encoder with specified parameters and creates required
@@ -259,7 +267,7 @@ class Encoder(nn.Module):
         self.d_model = d_model
 
         self.encoder_layers = nn.ModuleList(
-            [EncoderLayer(d_model, num_heads, dropout_rate, ff_factory, norm_factory)
+            [EncoderLayer(d_model, num_heads, dropout_rate, ff_factory, norm_factory, use_flash=use_flash)
              for _ in range(num_encoder_layers)]
         )
         self.final_layer_norm = norm_factory()
@@ -330,7 +338,9 @@ class Decoder(nn.Module):
             dropout_rate: float,
             ff_factory: Callable[[], nn.Module],
             norm_factory: Callable[[], nn.Module],
-            pad_token_id: int
+            pad_token_id: int,
+            use_flash: bool = False,
+            use_cross_attn: bool = False
     ) -> None:
         """
         A Decoder class that implements a multi-layer transformer decoder for sequence-to-sequence
@@ -361,7 +371,8 @@ class Decoder(nn.Module):
         self.d_model = d_model
 
         self.decoder_layers = nn.ModuleList(
-            [DecoderLayer(d_model, num_heads, dropout_rate, ff_factory, norm_factory)
+            [DecoderLayer(d_model, num_heads, dropout_rate, ff_factory,
+                          norm_factory, use_flash=use_flash, use_cross_attn=use_cross_attn)
              for _ in range(num_decoder_layers)]
         )
         self.final_layer_norm = norm_factory()
@@ -444,7 +455,8 @@ class Transformer(nn.Module):
             ff_factory: Callable[[], nn.Module],
             norm_factory: Callable[[], nn.Module],
             pad_token_id: int,
-            use_encoder: bool = True
+            use_encoder: bool = True,
+            use_flash: bool = False
     ) -> None:
         """
         Initializes the Transformer model composed of an encoder and a decoder with
@@ -468,13 +480,13 @@ class Transformer(nn.Module):
             self.encoder = Encoder(num_encoder_layers, d_model, num_heads,
                                    input_vocab_size, dropout_rate,
                                    ff_factory, norm_factory,
-                                   pad_token_id)
+                                   pad_token_id, use_flash=use_flash)
         else:
             self.encoder = None
         self.decoder = Decoder(num_decoder_layers, d_model, num_heads,
                                target_vocab_size, dropout_rate,
                                ff_factory, norm_factory,
-                               pad_token_id)
+                               pad_token_id, use_flash=use_flash, use_cross_attn=self.use_encoder)
         self.output_fc = nn.Linear(d_model, target_vocab_size, bias=False)
 
     def forward(
